==> coventry-v0.9.2.tar.gz: coventry-v0.9.2/utils/pbx-runstate.cpp <==
/*
 * Copyright (C) 2022 Tycho Softworks.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "server.hpp"       // IWYU pragma: keep
#include "ipc.hpp"

#include <moderncli/socket.hpp>
#include <moderncli/filesystem.hpp>
#include <moderncli/args.hpp>

#include <fcntl.h>

namespace {
args::option helpflag('h',  "--help",       "display this list");
args::option quiet('q',     "--quiet",      "suppress error output");
args::option althelp('?',   nullptr, nullptr);

class ipc_error final : public std::exception {
public:
    explicit ipc_error(const char *text, int code) noexcept : msg(text), err(code) {}

    auto what() const noexcept -> const char * override {
        return msg;
    }

    auto exit() const noexcept {
        return err;
    }

private:
    const char *msg;
    int err;
};
} // end namespace

auto main(int argc, const char **argv) -> int {
    auto exit_code = 0;
    try {
        auto count = args::parse(argc, argv);
        if(is(helpflag) || is(althelp) || count > 1) {
            args::help({"pbx-runstate [new-state]"}, {
                "pbx-runstate " PROJECT_VERSION,
                "David Sugar <tychosoft@gmail.com",
                "Change pbx running mode"
            });
            return 0;
        }

        fsys::current_path(SERVER_PREFIX_PATH);
        if(!count) {
            std::error_code error;
            auto symlink = fsys::read_symlink("state.conf", error);
            if(!symlink.empty() && fsys::is_regular_file(symlink)) {
                const auto filename = symlink.filename().string();
                const auto path = symlink.string();
                if (filename[0] != '.' && path == filename) {
                    print("{}\n", symlink.stem().string());
                    ::exit(0);
                }
            }
            print("default\n");
            ::exit(0);
        }

        auto state = std::string{args::list()[0]};
        if(state == "default") {
            fsys::remove("state.conf");
        }
        else {
            state += ".state";
            if(!fsys::is_regular_file(state))
                throw std::runtime_error(format("{}: unknown state", state));
            fsys::remove("state.conf");
            fsys::create_symlink(state, "state.conf");
        }

        const pbx_msg msg{.type = PBX_RELOAD, .ver = PBX_VERSION};
        auto ipc(Socket(AF_UNIX, SOCK_DGRAM));
        if(!ipc)
            throw ipc_error("control failed", 4);

        send(ipc, msg, address_t(SERVER_CONTROL_IPC));
        ::exit(0);
    }
    catch(const bad_arg& e) {
        die(-1, "Usage: pbx-runstate [new-state]\nerror: {}\n", e.what());
    }
    catch(const ipc_error& e) {
        exit_code = e.exit();
        if(!is(quiet))
            die(exit_code, "error: {}\n", e.what());
    }
    catch(const std::exception& e) {
        if(is(quiet))
            exit_code = -1;
        else
            die(-1, "error: {}\n", e.what());
    }
    catch(...) {
        if(is(quiet))
            exit_code = -1;
        else
            die(-1, "error: unknown error\n");
    }
    return exit_code;
}

==> coventry-v0.9.3.tar.gz: coventry-v0.9.3/utils/pbx-runstate.cpp <==
/*
 * Copyright (C) 2022 Tycho Softworks.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "server.hpp"       // IWYU pragma: keep
#include "ipc.hpp"

#include <moderncli/socket.hpp>
#include <moderncli/filesystem.hpp>
#include <moderncli/args.hpp>

#include <fcntl.h>

namespace {
args::option helpflag('h',  "--help",       "display this list");
args::option quiet('q',     "--quiet",      "suppress error output");
args::option althelp('?',   nullptr, nullptr);

class ipc_error final : public std::exception {
public:
    explicit ipc_error(const char *text, int code) noexcept :
    msg_(text), err_(code) {}

    auto what() const noexcept -> const char * override {
        return msg_;
    }

    auto exit() const noexcept {
        return err_;
    }

private:
    const char *msg_;
    int err_;
};
} // end namespace

auto main(int argc, const char **argv) -> int {
    auto exit_code = 0;
    try {
        auto count = args::parse(argc, argv);
        if(is(helpflag) || is(althelp) || count > 1) {
            args::help({"pbx-runstate [new-state]"}, {
                "pbx-runstate " PROJECT_VERSION,
                "David Sugar <tychosoft@gmail.com",
                "Change pbx running mode"
            });
            return 0;
        }

        fsys::current_path(SERVER_PREFIX_PATH);
        if(!count) {
            std::error_code err;
            auto symlink = fsys::read_symlink("state.conf", err);
            if(!symlink.empty() && fsys::is_regular_file(symlink)) {
                const auto filename = symlink.filename().string();
                const auto path = symlink.string();
                if (filename[0] != '.' && path == filename) {
                    print("{}\n", symlink.stem().string());
                    ::exit(0);
                }
            }
            print("default\n");
            ::exit(0);
        }

        auto state = std::string{args::list()[0]};
        if(state == "default") {
            fsys::remove("state.conf");
        }
        else {
            state += ".state";
            if(!fsys::is_regular_file(state)) throw error(format("{}: unknown state", state));
            fsys::remove("state.conf");
            fsys::create_symlink(state, "state.conf");
        }

        const pbx_msg msg{.type = PBX_RELOAD, .ver = PBX_VERSION};
        auto ipc(Socket(AF_UNIX, SOCK_DGRAM));
        if(!ipc) throw ipc_error("control failed", 4);
        send(ipc, msg, address_t(SERVER_CONTROL_IPC));
        ::exit(0);
    }
    catch(const bad_arg& e) {
        die(-1, "Usage: pbx-runstate [new-state]\nerror: {}\n", e.what());
    }
    catch(const ipc_error& e) {
        if(is(quiet)) return e.exit();
        die(e.exit(), "error: {}\n", e.what());
    }
    catch(const std::exception& e) {
        if(is(quiet)) return -1;
        die(-1, "error: {}\n", e.what());
    }
    catch(...) {
        if(is(quiet)) return -1;
        die(-1, "error: unknown error\n");
    }
    return exit_code;
}
